/*
 * ---------------------------------------------------------------------------
 * This file includes all global declarations for the gasmixer HW/SW. 
 * Especially all ports used by the gasmixer are mapped to the underlaying 
 * C architecture. 
 * author : Mstl Georg
 * date   : 16.11.2006
 * ---------------------------------------------------------------------------
 */

 
#include <avr/io.h>


#ifndef __GASMIXER_DEFS__
	#define __GASMIXER_DEFS__
	
	/* 
	 * -------------------------------------------------------------------------
	 * generic port declarations used for LCD
	 * -------------------------------------------------------------------------
	 */
	#define LCD_PORT			PORTA
	#define	LCD_PIN				PINA
	#define	LCD_DDR				DDRA
	
	#define	LCD_DATA_0		0		//4 bit databus used
	#define	LCD_DATA_1 		1
	#define LCD_DATA_2 		2
	#define	LCD_DATA_3		3
	#define	LCD_EN				4		//enable
	#define LCD_RS		  	5		//register select
	#define	LCD_RW				6		//read or write operation
	
	
	/* 
	 * -------------------------------------------------------------------------
	 * generic port declarations used for button matrix
	 * -------------------------------------------------------------------------
	 */
	#define BUT_PORT		PORTD
	#define BUT_PIN			PIND
	#define BUT_DDR 		DDRD
		
	#define BUT_COL_0		0
	#define BUT_COL_1		1
	#define BUT_COL_2		2
	#define BUT_LIN_0		3
	#define BUT_LIN_1		4
	#define BUT_LIN_2		5
	#define BUT_LIN_3		6
	
	
	/* 
	 * -------------------------------------------------------------------------
	 * general type declarations
	 * -------------------------------------------------------------------------
	 */
	typedef struct {
	
		//concret gas is characterized through[%]O2, [%]He, [%]N2
		uint8_t per_O2;
		uint8_t per_He;
		uint8_t per_N2;
	
	} gas_mix_t;
	
	typedef struct {
	
		//amount of O2, He, Top gas [bar] to get specified gasmix
		uint8_t am_O2;
		uint8_t am_He;
		uint8_t am_Top;
		
	} gas_amount_t;
	
	typedef struct {
	
		uint8_t  p_store_tank[2]; //pressure of storing tank [bar]
															//idx 0 -> O2 storing tank
															//idx 1 -> He storing tank
		uint8_t  V_store_tank[2]; //volume of storing tank [l]
															//idx 0 -> O2 storing tank
															//idx 1 -> He storing tank
		uint8_t  V_dive_tank;  //- " -  of diving tank [l]
	  gas_mix_t topoff;      //topoff gas in [%]02, [%]He, [%]N2
		uint8_t  p_off_comp;   //shutoff pressure of compressor [bar]
		
	} cfg_par_t;
	
	typedef struct {
	
		gas_mix_t gas; //actual gas in the tank [O2, He, N2] [%]
		uint8_t  p;    //actual pressure in the tank [bar]
	
	} tank_t;
	
	typedef struct {
	
	  gas_amount_t amount; //needed amount of O2, He, Top [bar]
		uint8_t p;           //needed pressure in the tank [bar]
		
	} fill_instr_t;
	
	
	/* 
	 * -------------------------------------------------------------------------
	 * general constant declarations
	 * -------------------------------------------------------------------------
	 */
	#define P_STORING_TANK	200	//standard pressure of storing tank [bar]
	#define P_DIVING_TANK   200 //- " -                diving tank [bar]
	#define V_STORING_TANK   50 //standard volume of storing tank [l]
	#define V_DIVING_TANK    12 //- " -              diving tank [l]   
  #define TOPOFF_O2        21 //O2 part in topoff gas [%]
  #define TOPOFF_HE         0 //He part - " -
	#define TOPOFF_N2        79 //N2 part - " -
	#define P_OFF_COMP      200 //standard shutoff pressure of compressor [bar]
 
	#define MAX_VAL_GAS     100 //max. value of one ingredient of gas [%]
	#define MAX_VAL_P_STORE 200 //max. pressure of storing tank [bar]
	#define MAX_VAL_P_DIVE  200 //- " -            diving tank [bar]
	#define MAX_VAL_P_COMP  200 //max. shutoff pressure of compressor [bar]
	#define MAX_VAL_V_STORE  50 //max. volume of storing tank [l]
	#define MAX_VAL_V_DIVE   12 //- " -          diving tank [l]
	
	#define MIN_VAL_GAS       0 //min. value of one ingredient of gas [%]
	#define MIN_VAL_P_STORE   2 //min. pressure of storing tank [bar]
	#define MIN_VAL_P_DIVE    0 //- " -            diving tank [bar]
	#define MIN_VAL_P_COMP    2 //min. shutoff pressure of compressor [bar]
	#define MIN_VAL_V_STORE  50 //min. volume of storing tank [l]
	#define MIN_VAL_V_DIVE   10 //- " -          diving tank [l]
	
	#define IDX_O2						0
	#define IDX_HE						1
	#define IDX_N2						2
	
	/* 
	 * -------------------------------------------------------------------------
	 * general function declaration
	 * -------------------------------------------------------------------------
	 */
	
	
	
#endif
