/*
 * ---------------------------------------------------------------------------
 * This file defines constants, functions, ... used to control/interpret the
 * pressed buttons at the matrix button field.
 * author : Mstl Georg
 * date   : 16.11.2006
 * ---------------------------------------------------------------------------
 */


#include "keypad_ctrl.h" 
#include "gasmixer_defs.h"
 
#include <util/delay.h>	//_delay_ms()
#include <stdbool.h>
 
#include <stdio.h>

#include "lcd_ctrl.h"
 	
	
/* 
 * -------------------------------------------------------------------------
 * internal function declarations 
 * -------------------------------------------------------------------------
 */

uint8_t detColIdxKEY(const uint8_t diff);

void setPortHighKEY(void);

void setPortDirKEY(void);
	 
	 
/* 
 * -------------------------------------------------------------------------
 * internal function definitions
 * -------------------------------------------------------------------------
 */
 
uint8_t detColIdxKEY(const uint8_t diff) {

  switch (diff) {
		case (1<<BUT_COL_0) : return 0; 
		case (1<<BUT_COL_1) : return 1; 
		case (1<<BUT_COL_2) : return 2; 
		default : return 255;
  }
	
}//detColIdxKEY


void setPortDirKEY(void) {

  //set columns to input
	BUT_DDR &= ~ ( (1<<BUT_COL_0) |
	               (1<<BUT_COL_1) |
								 (1<<BUT_COL_2) );
	
	//set lines to output
	BUT_DDR |= ( (1<<BUT_LIN_0) |
							 (1<<BUT_LIN_1) |
							 (1<<BUT_LIN_2) |
							 (1<<BUT_LIN_3) );
	
}//setPortDirKEY


void setPortHighKEY(void) {

	//setting port for input means activate internal pull up resistor
	BUT_PORT |= ( (1<<BUT_COL_0) |
	              (1<<BUT_COL_1) |
								(1<<BUT_COL_2) |
								(1<<BUT_LIN_0) |
							  (1<<BUT_LIN_1) |
							  (1<<BUT_LIN_2) |
							  (1<<BUT_LIN_3) );

}//setPortHighKEY

	 
/* 
 * -------------------------------------------------------------------------
 * general function definitions
 * -------------------------------------------------------------------------
 */
 
void initKEY(void) {

	setPortHighKEY();
	setPortDirKEY();

}//initKEY
 
  
uint8_t readKeypadKEY(volatile uint8_t* port, volatile uint8_t* pin) {

  const uint8_t col_idle = ( (1<<BUT_COL_0) |
												     (1<<BUT_COL_1) |
												     (1<<BUT_COL_2) );
  const uint8_t lin_offset = 3;	
  const uint8_t lines[] = {BUT_LIN_0, BUT_LIN_1, BUT_LIN_2, BUT_LIN_3};		 		
	
	uint8_t lin, col;
	
	volatile uint8_t sample_1;
  volatile uint8_t sample_2;

	uint8_t i;
	
	//loop through lines -> for every line read column input
	for(lin=0; lin<4; lin++) {
	
		//set actual selected line to 0, other lines to 1
		setPortHighKEY();
		//BUT_PORT &= ~ (1<<lines[lin]);
		BUT_PORT &= ~ (1<<lines[3]);
		
		//read pin value 
		//sample_1 = *pin;
		sample_1 = BUT_PIN;
	  
		sendTextPosLCD("BUT_PIN: ", SECOND_LINE_START_LCD);
		for (i=0; i<8; i++) {
			if (BUT_PIN & (1<<i))
				sendTextPosLCD("1", SECOND_LINE_START_LCD + 10 + i);
			else
			  sendTextPosLCD("0", SECOND_LINE_START_LCD + 10 + i);
		}
		
		sendTextPosLCD("sample_1: ", THIRD_LINE_START_LCD);
		for (i=0; i<8; i++) {
			if (sample_1 & (1<<i))
				sendTextPosLCD("1", THIRD_LINE_START_LCD + 10 + i);
			else
			  sendTextPosLCD("0", THIRD_LINE_START_LCD + 10 + i);
		}
		sample_1 &= ( (1<<BUT_COL_0) |
		              (1<<BUT_COL_1) |
							    (1<<BUT_COL_2) );
		//check if button was pressed 
		//-> when pressed input is set to 0
		//-> if more than one button in the actual line is pressed this is handled
		//like no button was pressed
		if((col = detColIdxKEY(col_idle - sample_1)) < 255) {
		
			//debounce button -> wait some time then check button again
			_delay_ms(SAMPLE_TIME);
			//sample_2 = *pin; 
			sample_2 = BUT_PIN;
			sample_2 &= ( (1<<BUT_COL_0) |
							    	(1<<BUT_COL_1) |
								    (1<<BUT_COL_2) );
				
			//check if button was pressed second time
			if(sample_1 == sample_2) 
        return lin * lin_offset + col;
			
		}//if
		
	}//for
	
	return NO_KEY;

}//readKeypadKEY
 
 
void waitReleasedKEY(uint8_t button) {

  const uint8_t lin_offset = 3;
  const uint8_t lines[] = {BUT_LIN_0, BUT_LIN_1, BUT_LIN_2, BUT_LIN_3};	
  const uint8_t cols[]  = {BUT_COL_0, BUT_COL_1, BUT_COL_2};						
	
	uint8_t lin, col;
	uint8_t sample_1, sample_2;
	bool but_rel = false;
	
	//determine line and col index
	//line has to be multiple of line offset (== 3)
	for(lin=1; lin<=4 && (button - lin * lin_offset > 0); lin++);

	col = button - (--lin)*lin_offset;
	
	//check specified button (lin, col)
	//set actual selected line to 0
	setPortHighKEY();
	BUT_PORT &= ~ (1<<lines[lin]);

  while(!but_rel) {
	
		//read pin value 
		sample_1 = BUT_PIN;
		sample_1 &= (1<<cols[col]);
	
		//button released -> port value '1'
		if(sample_1) {
	
			_delay_ms(100);
			sample_2 = BUT_PIN;
			sample_2 &= (1<<cols[col]);
				
			//check if button was pressed second time
			if(sample_1 == sample_2) 
        but_rel = true;			
	
	  }//if
		
	}//while
	
}//waitReleasedKEY


void waitNoKEY(void) {

	const uint8_t col_idle = ( (1<<BUT_COL_0) |
												     (1<<BUT_COL_1) |
												     (1<<BUT_COL_2) );
  const uint8_t lines[] = {BUT_LIN_0, BUT_LIN_1, BUT_LIN_2, BUT_LIN_3};			
	
	uint8_t lin, h;
	uint8_t sample;
	
	bool idle       = false;
	bool second_run = false;
	bool idle_lin[] = {false, false, false, false};
	
	//loop through lines -> for every line read column input
  while(!idle) {
		
		for(lin=0; lin<4 && !idle; lin++) {
	
			//set actual selected line to 0
			BUT_PORT &= ~ (1<<lines[lin]);
		
			//read pin value 
			sample = BUT_PIN;
			sample &= ( (1<<BUT_COL_0) |
						  		(1<<BUT_COL_1) |
									(1<<BUT_COL_1) );
		
			//check if no button was pressed 
			if(sample == col_idle) {
			
				//set idle flag of line true
				idle_lin[lin] = true;
				
		  } else {
			
				idle_lin[lin] = false;
				second_run    = false;
			
			}
				
			if(idle_lin[0] && idle_lin[1] && idle_lin[2] && idle_lin[3] && 
			   second_run) { 
				 
				 idle = true;
			
			} else if(idle_lin[0] && idle_lin[1] && idle_lin[2] && idle_lin[3]) {
			
				//all lines are idle -> wait and take second sample
				_delay_ms(100);
				second_run = true;
				for(h=0; h<4; h++)
				  idle_lin[h] = false;
					
			}//if	
		
		}//for
		
  }//while	
		
}//waitNoKEY

	
uint8_t detKeyValKEY(uint8_t key) {

	uint8_t res = 0;

	switch(key) {
	
		case BUT_0: 
			res = 0; break;
		
		case BUT_1: case BUT_2: case BUT_3: case BUT_4: case BUT_5: 
		case BUT_6: case BUT_7: case BUT_8: case BUT_9:
			res = key + 1; break;
			
	  case BUT_FENCE: case BUT_MULT:
			res = key; break;
			
	  default: //NO_KEY
		  res = NO_KEY;

  }//switch
	
	return res;

}//detKeyValKEY	
	
	
	
